package com.qmth.paper.library.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.paper.library.business.entity.PaperLibrary;
import com.qmth.paper.library.business.entity.PaperScanTask;
import com.qmth.paper.library.business.service.ClientService;
import com.qmth.paper.library.business.service.PaperLibraryCommonService;
import com.qmth.paper.library.business.service.PaperLibraryService;
import com.qmth.paper.library.business.service.PaperScanTaskService;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.SysPrivilege;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.enums.UploadFileEnum;
import com.qmth.paper.library.common.lock.LockService;
import com.qmth.paper.library.common.lock.LockType;
import com.qmth.paper.library.common.service.SysPrivilegeService;
import com.qmth.paper.library.common.util.ServletUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 客户端 服务实现类
 * </p>
 */
@Service
public class ClientServiceImpl implements ClientService {

    @Resource
    PaperScanTaskService paperScanTaskService;

    @Resource
    SysPrivilegeService sysPrivilegeService;

    @Resource
    PaperLibraryService paperLibraryService;

    @Resource
    PaperLibraryCommonService paperLibraryCommonService;

    @Resource
    LockService lockService;

    @Override
    public IPage<PaperScanTask> pageScanTask(Long paperArchivesId, String courseName, String teachClazzName, Boolean isMine, Integer pageNumber, Integer pageSize) {
        Long schoolId = SystemConstant.convertIdToLong(ServletUtil.getRequestHeaderSchoolId().toString());
        // 查询任务
        QueryWrapper<PaperScanTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PaperScanTask::getSchoolId, schoolId);
        if (paperArchivesId != null) {
            queryWrapper.lambda().eq(PaperScanTask::getPaperArchivesId, paperArchivesId);
        }
        if (StringUtils.isNotBlank(courseName)) {
            queryWrapper.lambda().eq(PaperScanTask::getCourseName, courseName);
        }
        if (StringUtils.isNotBlank(teachClazzName)) {
            queryWrapper.lambda().eq(PaperScanTask::getTeachClazzName, teachClazzName);
        }
        if (isMine != null) {
            // 我的任务
            if (isMine) {
                Long userId = Long.valueOf(ServletUtil.getRequestHeaderUserId().toString());
                queryWrapper.lambda().eq(PaperScanTask::getScanUserId, userId);
            } else {
                queryWrapper.lambda().isNull(PaperScanTask::getScanUserId);
            }
        }
        return paperScanTaskService.page(new Page<>(pageNumber, pageSize), queryWrapper);
    }

    @Override
    public void checkPrivilege(Long userId) {
        List<SysPrivilege> sysPrivileges = sysPrivilegeService.getClientUrlByUserId(userId);
        long count = sysPrivileges.stream().filter(m -> "client".equalsIgnoreCase(m.getUrl())).count();
        if (count == 0) {
            throw ExceptionResultEnum.ERROR.exception("用户没有权限，无法登录");
        }
    }

    @Override
    public boolean bindUser(Long paperScanTaskId, Long userId) {
        try {
            lockService.waitlock(LockType.BIND_SCAN_USER, paperScanTaskId);
            PaperScanTask paperScanTask = paperScanTaskService.getById(paperScanTaskId);
            if (paperScanTask == null) {
                throw ExceptionResultEnum.ERROR.exception("扫描任务不存在");
            }

            if (paperScanTask.getEnable()) {
                throw ExceptionResultEnum.ERROR.exception("扫描任务已关闭");
            }

            // 扫描任务是否被其它人扫描
            if (paperScanTask.getScanUserId() != null && !paperScanTask.getScanUserId().equals(userId)) {
                throw ExceptionResultEnum.ERROR.exception("其它扫描员已开始扫描");
            }

            if (userId == null) {
                SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
                userId = sysUser.getId();
            }

            UpdateWrapper<PaperScanTask> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(PaperScanTask::getScanUserId, userId).eq(PaperScanTask::getId, paperScanTaskId);
            return paperScanTaskService.update(updateWrapper);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("扫描任务错误：" + e.getMessage());
        } finally {
            lockService.unlock(LockType.BIND_SCAN_USER, paperScanTaskId);
        }
    }

    @Transactional
    @Override
    public boolean pictureUpload(Long paperScanTaskId, MultipartFile frontFile, String frontMd5, MultipartFile versoFile, String versoMd5) {
        Long userId = Long.valueOf(ServletUtil.getRequestHeaderUserId().toString());
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        if (Objects.isNull(frontFile) || frontFile.getSize() == 0) {
            throw ExceptionResultEnum.ERROR.exception("frontFile为空");
        }
        if (Objects.isNull(versoFile) || versoFile.getSize() == 0) {
            throw ExceptionResultEnum.ERROR.exception("versoFile为空");
        }

        try {
            String frontFileMd5 = DigestUtils.md5Hex(frontFile.getBytes());
            if (!Objects.equals(frontMd5, frontFileMd5)) {
                throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
            }
            String versoFileMd5 = DigestUtils.md5Hex(versoFile.getBytes());
            if (!Objects.equals(versoMd5, versoFileMd5)) {
                throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
            }

            // 保存图片，生成路径
            String path = paperLibraryCommonService.saveLibraryFile(UploadFileEnum.IMAGE, frontFile, versoFile);

            PaperLibrary paperLibrary = new PaperLibrary();
            paperLibrary.setId(SystemConstant.getDbUuid());
            paperLibrary.setSchoolId(schoolId);
            paperLibrary.setPaperScanTaskId(paperScanTaskId);
            paperLibrary.setPath(path);
            paperLibrary.setAbnormal(false);
            paperLibrary.setCreateId(userId);
            paperLibrary.setCreateTime(System.currentTimeMillis());
            paperLibraryService.save(paperLibrary);

            // 统计扫描数量
            int count = paperLibraryService.countScanCount(paperScanTaskId);
            UpdateWrapper<PaperScanTask> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(PaperScanTask::getScanCount, count)
                    .eq(PaperScanTask::getId, paperScanTaskId);
            paperScanTaskService.update(updateWrapper);
            return true;

        } catch (IOException e) {
            throw ExceptionResultEnum.ERROR.exception("上传图片失败：" + e.getMessage());
        }
    }

    @Override
    public boolean enable(Long paperScanTaskId, Boolean enable) {
        return paperScanTaskService.enable(paperScanTaskId, enable);
    }
}
