package com.qmth.paper.library.api;


import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.paper.library.business.bean.result.EditResult;
import com.qmth.paper.library.business.service.PaperLibraryCommonService;
import com.qmth.paper.library.business.templete.execute.AsyncStudentDataImportService;
import com.qmth.paper.library.common.bean.params.BasicStudentParams;
import com.qmth.paper.library.common.contant.ApiPrefixConstant;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.entity.TBTask;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.enums.TaskTypeEnum;
import com.qmth.paper.library.common.service.BasicStudentService;
import com.qmth.paper.library.common.util.Result;
import com.qmth.paper.library.common.util.ResultUtil;
import com.qmth.paper.library.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 学校id 前端控制器
 * </p>
 */
@Api(tags = "学生管理controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + ApiPrefixConstant.PREFIX_BASIC + "/student")
@Validated
public class BasicStudentController {
    @Resource
    private BasicStudentService basicStudentService;
    @Resource
    private PaperLibraryCommonService paperLibraryCommonService;
    @Resource
    private AsyncStudentDataImportService asyncStudentDataImportService;

    @ApiOperation(value = "学生基本信息管理-查询")
    @RequestMapping(value = "/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result findBasicStudentList(@ApiParam(value = "查询参数（学生学号或姓名）") @RequestParam(required = false) String queryParams,
                                       @ApiParam(value = "学院主键") @RequestParam(required = false) String collegeId,
                                       @ApiParam(value = "专业名称") @RequestParam(required = false) String majorName,
                                       @ApiParam(value = "班级名称") @RequestParam(required = false) String clazzName,
                                       @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                       @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(basicStudentService.basicStudentPage(queryParams, SystemConstant.convertIdToLong(collegeId), majorName, clazzName, pageNumber, pageSize));
    }

    @ApiOperation(value = "学生基本信息管理-新增/编辑")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "更新成功", response = EditResult.class)})
    public Result saveBasicStudent(@Valid @RequestBody BasicStudentParams basicStudentParams, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(basicStudentService.saveBasicStudent(basicStudentParams, sysUser));
    }

    @ApiOperation(value = "学生基本信息管理-批量删除（物理）")
    @RequestMapping(value = "/delete_batch", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "删除成功", response = EditResult.class)})
    public Result deleteBasicStudent(@RequestParam String ids) {
        String[] idsArray = ids.split(",");
        List<Long> longList = Stream.of(idsArray).map(Long::parseLong).collect(Collectors.toList());
        return ResultUtil.ok(basicStudentService.removeBasicStudentBatch(longList));
    }

    @ApiOperation(value = "学生基本信息管理-根据条件删除（物理）")
    @RequestMapping(value = "/delete_by_condition", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result deleteBasicStudentByCondition(@ApiParam(value = "查询参数（学生学号或姓名）") @RequestParam(required = false) String queryParams,
                                                @ApiParam(value = "学院主键") @RequestParam(required = false) String collegeId,
                                                @ApiParam(value = "专业名称") @RequestParam(required = false) String majorName,
                                                @ApiParam(value = "班级名称") @RequestParam(required = false) String clazzName) {

        if (!SystemConstant.strNotNull(queryParams) && !SystemConstant.strNotNull(collegeId) &&
                !SystemConstant.strNotNull(majorName) && !SystemConstant.strNotNull(clazzName)) {
            throw ExceptionResultEnum.ERROR.exception("请选择要删除的条件");
        }
        return ResultUtil.ok(basicStudentService.removeBasicStudentByCondition(queryParams, SystemConstant.convertIdToLong(collegeId), majorName, clazzName));
    }

    @ApiOperation(value = "学生基本信息管理-批量导入（异步）")
    @RequestMapping(value = "/data_import", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    public Result basicStudentImportAsync(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file) throws Exception {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Map<String, Object> map = paperLibraryCommonService.saveTask(file, TaskTypeEnum.STUDENT_IMPORT);
        map.put(SystemConstant.SYS_USER, sysUser);

        asyncStudentDataImportService.importTask(map);
        TBTask tbTask = Objects.nonNull(map.get(SystemConstant.TASK)) ? (TBTask) map.get(SystemConstant.TASK) : null;
        return Objects.nonNull(tbTask) ? ResultUtil.ok(tbTask.getId()) : ResultUtil.error("创建任务失败");
    }

}
